package com.example.ellegadodepintia.exploradoresDePintia.model

import android.graphics.drawable.AnimationDrawable
import android.widget.ImageView
import android.widget.TextView
import com.example.ellegadodepintia.R
import com.example.ellegadodepintia.exploradoresDePintia.model.GameState.context
import com.example.ellegadodepintia.exploradoresDePintia.model.eventoFactory.Evento
import com.example.ellegadodepintia.exploradoresDePintia.model.eventoFactory.EventoCombate
import com.example.ellegadodepintia.exploradoresDePintia.model.eventoFactory.EventoFactorySelector


object GeneradorDeEventos {
    private const val CANTIDAD = 8 // Numero de eventos generados
    private var eventos = ArrayList<Evento>()

    private fun crearEvento(): Evento {
        val factory = EventoFactorySelector.obtenerFactory()
        return factory.generarEvento()
    }

    fun generarEventos() {
        val eventosGenerados = ArrayList<Evento>()
        while (eventosGenerados.size < CANTIDAD) {
            val nuevoEvento = crearEvento()

            if (!eventosGenerados.contains(nuevoEvento)) {
                eventosGenerados.add(nuevoEvento)
            }
        }
        eventos = eventosGenerados
    }

    fun comprobarEventoDisponible() : Boolean{
        return eventos.size  >= 1
    }

    fun generarEvento(dificultad: Int) {
        "Resultado del evento".also { context!!.findViewById<TextView>(R.id.textoResultado).text = it }

        val eventoSeleccionado = filtrarEventosPorDificultad(eventos, dificultad)
        eventos.remove(eventoSeleccionado)

        val textEvento = context!!.findViewById<TextView>(R.id.textoEvento)
        textEvento.text = eventoSeleccionado.descripcion
        val imagenEvento = context!!.findViewById<ImageView>(R.id.imagenEvento)

        imagenEvento.setImageResource(eventoSeleccionado.imagen)

        val drawable = imagenEvento.drawable

        if (drawable is AnimationDrawable && eventoSeleccionado is EventoCombate) {
            drawable.start()
        }

        if(eventoSeleccionado is EventoCombate){
            GestorCombate.evento = eventoSeleccionado
        }
        GameState.eventoActual = eventoSeleccionado
        GameState.eventoActual.addObserver {
            LayoutUtils.actualizarEstadisticasEvento() }

        LayoutUtils.actualizarEstadisticasEvento()

    }

    private fun filtrarEventosPorDificultad(eventos: ArrayList<Evento>, dificultad: Int): Evento {
        val eventosFiltrados = eventos.filter { it.dificultad == dificultad }
        return eventosFiltrados[eventosFiltrados.indices.random()]
    }


}
