package com.example.ellegadodepintia.exploradoresDePintia.model.eventoFactory

import com.example.ellegadodepintia.R
import com.example.ellegadodepintia.exploradoresDePintia.model.GameState
import com.example.ellegadodepintia.exploradoresDePintia.model.Atributo
import com.example.ellegadodepintia.exploradoresDePintia.model.Opcion
import com.example.ellegadodepintia.exploradoresDePintia.model.OpcionCombate
import com.example.ellegadodepintia.exploradoresDePintia.model.efecto.EfectoAtaqueEnemigo
import com.example.ellegadodepintia.exploradoresDePintia.model.efecto.EfectoAtaqueProtagonista
import com.example.ellegadodepintia.exploradoresDePintia.model.efecto.EfectoHabilidadesProtagonista
import com.example.ellegadodepintia.exploradoresDePintia.model.efecto.EfectoHuirProtagonista
import com.example.ellegadodepintia.exploradoresDePintia.model.efecto.EfectoInvestigarEnemigo

class CombateFactory : EventoFactory {

    private val combatesMap = mapOf(
        "El Metal Volador es una criatura formada por fragmentos de metal antiguo. Protege los secretos del yacimiento de Pintia." to Triple(
            R.drawable.monster_metal_idle_animation,
            R.drawable.monster_metal_attack_animation,
            listOf(
                OpcionCombate(
                    descripcion = "Atacar",
                    atributoRequerida = Atributo.Ataque,
                    efecto = EfectoAtaqueProtagonista(70)
                ),
                Opcion(
                    descripcion = "Investigar",
                    atributoRequerida = Atributo.Investigacion,
                    EfectoInvestigarEnemigo(100)
                ),
                Opcion(
                    descripcion = "Habilidades",
                    atributoRequerida = Atributo.Nula,
                    efecto = EfectoHabilidadesProtagonista(GameState.jugador.atributos[Atributo.Vida]!!)
                ),
                OpcionCombate(
                    descripcion = "Huir",
                    atributoRequerida = Atributo.Vida,
                    efecto = EfectoHuirProtagonista(GameState.jugador.atributos[Atributo.Vida]!!),
                ),
            )
        ),
    )

    override fun generarDescripcion(): String {
        return combatesMap.keys.random()
    }

    override fun generarEvento(): EventoCombate {
        val descripcion = generarDescripcion()
        return EventoCombate(
            descripcion = descripcion,
            dificultad = 0,
            opciones = generarOpciones(descripcion),
            imagenAtaque = generarImagenAtaque(descripcion),
            imagen = generarImagen(descripcion),
            resistenciaMax = (5..10).random(),
            ataque = (1..2).random(),
            finalizado = false,
            acciones = mutableListOf(EfectoAtaqueEnemigo(70))
        )
    }

    override fun generarOpciones(descripcion: String): MutableList<Opcion> {
        return combatesMap[descripcion]?.third?.take(4)?.toMutableList()
            ?: mutableListOf()
    }

    override fun generarImagen(descripcion: String): Int {
        return combatesMap[descripcion]?.first!!
    }

    private fun generarImagenAtaque(descripcion: String): Int {
        return combatesMap[descripcion]?.second!!
    }

}