package com.example.ellegadodepintia.exploradoresDePintia.model.eventoFactory

import com.example.ellegadodepintia.exploradoresDePintia.model.TipoEvento

object EventoFactorySelector {

    fun obtenerFactory(): EventoFactory {
        val tipo = seleccionarTipoEventoConProbabilidades()

        return when (tipo) {
            TipoEvento.Reliquia -> ReliquiaFactory()
            TipoEvento.Combate -> CombateFactory()
            TipoEvento.Tienda -> TiendaFactory()
            TipoEvento.Trampa -> TrampaFactory()
            TipoEvento.Riesgo -> RiesgoFactory()
            TipoEvento.Descanso -> DescansoFactory()
            TipoEvento.Ladron -> LadronFactory()
            TipoEvento.Desenterrar -> DesenterrarFactory()
        }
    }

    private fun seleccionarTipoEventoConProbabilidades(): TipoEvento {
        val probabilidades = mapOf(
            TipoEvento.Reliquia to 0.2,
            TipoEvento.Tienda to 0.1,
            TipoEvento.Riesgo to 0.1,
            TipoEvento.Trampa to 0.1,
            TipoEvento.Combate to 0.2,
            TipoEvento.Descanso to 0.1,
            TipoEvento.Ladron to 0.1,
            TipoEvento.Desenterrar to 0.1
        )

        val acumulado = probabilidades.entries.fold(mutableListOf<Pair<Double, TipoEvento>>()) { acc, entry ->
            val sumaAcumulada = acc.lastOrNull()?.first ?: 0.0
            acc.add(Pair(sumaAcumulada + entry.value, entry.key))
            acc
        }

        val random = Math.random()
        return acumulado.first { random <= it.first }.second
    }
}
